@@ -1,10 +1,8 @@
-'''
-Constantin Werner 01.09.2020
-const.werner@gmail.com
-'''
-
 import re
 import pymorphy2
+import nltk
+nltk.download('punkt')
+from nltk.tokenize import word_tokenize
 
 class Separatrice:
   def __init__(self):
@@ -49,13 +47,29 @@ class Separatrice:
     sentences = [s.strip() for s in sentences if s not in ["!","?",'.']]
     return sentences
   
+  def introductory_phrase(self, sent):
+    tokens = word_tokenize(sent)
+    comma = False
+    i = 0
+    tag = self.morph.parse(tokens[i])[0].tag
+    while (('NOUN' in tag or 'ADJS' in tag or 'ADJF' in tag or 'ADJS' in tag or 'NPRO' in tag) or tokens[i] == ','):
+        if (tokens[i] == ','):
+          comma = True
+          break
+        i += 1
+        tag = self.morph.parse(tokens[i])[0].tag
+
+    if comma:    
+      return ' '.join(tokens[:i])
+    else:
+      return '' 
+  
   # check whether or not text contains predicate
-  def _pred_in(self,text):
+  def _pred_in(self,text,subj_required=False):
     '''
     params
     ---
     text : str
-
     return
     ---
     bool
@@ -65,53 +79,61 @@ class Separatrice:
     pron = False
     adj = False
     prt = False
+    verb = False
     for word in tokenized:
       word = word.strip('.')
       word = word.strip('!')
       word = word.strip('?')
       word = word.strip(',')
-      if 'NOUN' in self.morph.parse(word)[0].tag:
-        noun = True
-      if 'NPRO' in self.morph.parse(word)[0].tag:
+      if ('VERB' in self.morph.parse(word)[0].tag or 'INFN' in self.morph.parse(word)[0].tag or 'PRED' in self.morph.parse(word)[0].tag
+            or 'нет' == word or 'здесь' == word or 'тут' == word):
+          return True
+      if ('это' == word):
         pron = True
-      elif 'ADJF' in self.morph.parse(word)[0].tag or 'ADJS' in self.morph.parse(word)[0].tag:
+      elif ('ADJF' in self.morph.parse(word)[0].tag or 'ADJS' in self.morph.parse(word)[0].tag):
         adj = True
+      elif (len(self.morph.parse(word)) > 1):
+        if ('ADJF' in self.morph.parse(word)[1].tag or 'ADJS' in self.morph.parse(word)[1].tag):
+          adj = True
+        elif 'NOUN' in self.morph.parse(word)[0].tag:
+          noun = True
+      elif 'NOUN' in self.morph.parse(word)[0].tag and 'nomn' in self.morph.parse(word)[0].tag:
+        noun = True
+      elif 'NPRO' in self.morph.parse(word)[0].tag:
+        pron = True
       elif 'PRTF' in self.morph.parse(word)[0].tag or 'PRTS' in self.morph.parse(word)[0].tag:
         prt = True
-      elif 'VERB' in self.morph.parse(word)[0].tag or 'INFN' in self.morph.parse(word)[0].tag or 'PRED' in self.morph.parse(word)[0].tag:
-        return True
       if len(self.morph.parse(word)) > 1:
-        if 'VERB' in self.morph.parse(word)[1].tag:
+        if ('VERB' in self.morph.parse(word)[1].tag):
           return True
-    if ((noun == True or pron == True) and adj == True):
-      return True
-    if ((noun == True or pron == True) and prt == True):
+
+    if ((noun == True or pron == True) and (adj == True or prt == True or verb == True)):
       return True
     if (noun == True and pron == True):
       return True
+
     return False
   
-  # split by comma and check which pieces are true clauses 
-  def split_by_comma(self,text):
+  # split by delim and check which pieces are true clauses 
+  def separate_by(self,delim,text,subj_required=False):
     '''
     params
     ---
     text : str
-
     return
     ---
     result : list of str
     '''
     result = []
-    cands = [cand for cand in re.split(',',text) if cand != ' ']
+    cands = [cand for cand in re.split(delim,text) if cand != ' ']
     appended = [False]*len(cands)
     if (len(cands) > 1):
       for i in range(1,len(cands)):
         if self._pred_in(cands[i]) == False:
           if cands[i-1] in result:
             result.remove(cands[i-1])
-          result.append(cands[i-1] + ',' + cands[i])
-          cands[i] = cands[i-1] + ',' + cands[i]
+          result.append(cands[i-1] + delim + cands[i])
+          cands[i] = cands[i-1] + delim + cands[i]
           appended[i] = True
           appended[i-1] = True
         else:
@@ -121,7 +143,7 @@ class Separatrice:
         if self._pred_in(cands[0]) ==True:
           result.insert(0,cands[0])
         else:
-          result[0] = cands[0] + ',' + result[0]
+          result[0] = cands[0] + delim + result[0]
       return result
     result = cands
     return result
@@ -136,13 +158,35 @@ class Separatrice:
     ---
     clauses : list of str 
     '''
+    
+    # FIRSTLY CHECK INTRODUCTORY PHRASES, e.g. "Ребят, где тут мост?", "Привет, как там с деньгами?"
+    address_phrase = self.address_phrase(text)
+    if address_phrase != '':
+      text = re.sub(address_phrase, '', text)
+
     text = ' ' + text + ' '
     text = re.sub(', ' + self.conjs + ' | ' + self.conjs + ' ', '<stop>',text)
     clauses = text.split('<stop>')
     temp = []
-    for clause in clauses:
-      for x in self.split_by_comma(clause):
-        temp.append(x.strip(' '))
-    clauses = [x for x in temp if x != '']
+    if ';' in text:
+      temp = []
+      for clause in clauses:
+        for x in self.separate_by(';',clause):
+          temp.append(x.strip(' '))
+      clauses = [x for x in temp if x != '']
+    if ' - ' in text:
+      temp = []
+      for clause in clauses:
+        for x in self.separate_by('-',clause):
+          temp.append(x.strip(' '))
+      clauses = [x for x in temp if x != '']
+    if ',' in text:
+      for clause in clauses:
+        for x in self.separate_by(',',clause):
+          temp.append(x.strip(' '))
+      clauses = [x  for x in temp if x != '']
+    clauses = [clause.strip() for clause in clauses if clause.strip() != '']
+    if address_phrase != '':
+      clauses.insert(0,address_phrase)
     return clauses
 
